// line 295 "ModelEvaluator.jweb"
/*;
Copyright (c) 1998 The Omega Project for Statistical Computing.  
All rights reserved.

*/


// line 277 "ModelEvaluator.jweb"
   package org.omegahat.Simulation.MCMC.Expressions;


// line 283 "ModelEvaluator.jweb"
    import org.omegahat.Simulation.MCMC.*;
    import org.omegahat.Simulation.MCMC.Listeners.*;
    import org.omegahat.Simulation.MCMC.Proposals.*;
    import org.omegahat.Simulation.MCMC.Targets.*;

import org.omegahat.Models.Formulae.Expressions.*;
import org.omegahat.Environment.Interpreter.*;
import org.omegahat.Environment.Parser.Parse.*;
import org.omegahat.Environment.Databases.*;
import java.util.Enumeration;

// line 7 "ModelEvaluator.jweb"
public class ModelEvaluator 
          extends BasicEvaluator 
{


// line 33 "ModelEvaluator.jweb"
MultiLineModelFormula likelihoodModel;
MultiLineModelFormula proposalModel;

MCMCStateDatabase current;
MCMCStateDatabase proposed;
Database          definitions;

// Constants

static final int CURRENT  = 0;
static final int PROPOSAL = 1;

// line 230 "ModelEvaluator.jweb"
 protected List likelihood;


// line 22 "ModelEvaluator.jweb"
protected SearchPath databases( Database[] databases )
{
  for(int i =0; i < databases.length; i++)
    attach( databases[i], 0 );

  return this.databases;
}


// line 48 "ModelEvaluator.jweb"
protected MultiLineModelFormula likelihoodModel() 
{ 
return likelihoodModel; 
}

protected MultiLineModelFormula likelihoodModel( MultiLineModelFormula likelihoodModel) 
{ 
  this.likelihoodModel = likelihoodModel; 
  return likelihoodModel; 
}


public /*protected*/ MultiLineModelFormula proposalModel() 
{ 
return proposalModel; 
}

protected MultiLineModelFormula proposalModel( MultiLineModelFormula proposalModel) 
{ 
  this.proposalModel = proposalModel; 
  return proposalModel; 
}


// line 234 "ModelEvaluator.jweb"
public List likelihood() {
  return(likelihood);
}

public List likelihood(List l) {
  likelihood = l;
  return(likelihood());
}

// line 77 "ModelEvaluator.jweb"
public ModelEvaluator(MultiLineModelFormula likelihoodModel, 
		      MultiLineModelFormula proposalModel,
		      Database initialValues,
		      Database definitions ) throws Exception
{

  createAndAttachDatabases( initialValues, definitions );



  likelihood(createLikelihood(likelihoodModel)); // create expressions for terms in likelihood

  this.likelihoodModel = likelihoodModel;
  this.proposalModel = proposalModel;

  /* Only if we don't use objects, but actual functions
     for things such as N() and likelihood terms.

     addFunctionTable(DistributionMethods); 
   */
}


// line 102 "ModelEvaluator.jweb"
protected SearchPath createAndAttachDatabases( Database initialValues, Database  definitions)
{
  // create database to hold current state 
  current = createDatabase( "current", initialValues, true ); // copy

  // create database to hold the proposed state
  proposed = createDatabase( "proposed", initialValues, false ); // create null elements

  // keep a reference to the definitions
  this.definitions = definitions;

  // put references to each of these in the root database
  // assign( "current", current );
  // assign( "proposed", proposed );
  // assign( "definitions", definitions );

  // attach these in the order we want the search to occur
  // Scratch in position 0
  attach( current, 1 );
  attach( proposed, 2 ) ; 
  attach( definitions, 3 ) ;



  return searchPath();
}


/**;
 * Create a new database using another as a template.  Either
 * (shallow) copy contents or assign null elements of the same name as
 * initialValues 
 *;
 * @param name Name for the created database.
 * @param old  Database to use as the template
 * @param copy If true, copy elements.  If false, create null elements with same names.
 */
public MCMCStateDatabase createDatabase( String name, Database old, boolean copy) 
{
  // create database
  MCMCStateDatabase db = new MCMCStateDatabase( name );
  
  // get a list of what needs to be copied 
  String vars[] = old.objects(); 
  
  // iterate over variables copying or creating null elements
  for(int i=0; i < vars.length; i++) {
    /* Need to deal with names of length > 1 */
    try
      {
	if( copy )
	  db.assign( vars[i], old.get(vars[i]) );  // copy value
	else
	  db.assign( vars[i], Database.NULL_ENTRY); // empty value.
      }
    catch(Exception ex) 
      {
	ex.printStackTrace();
	throw new RuntimeException("Failed to assign into database in createDatabase.");
      }
	
  }

 return(db);
}

// line 245 "ModelEvaluator.jweb"
public List createLikelihood(MultiLineModelFormula model) {

 List expressions = new List(model.size());

  /* Create expressions whose values will be computed and 
    Multiplied in the eval()/nextStep()..... method at each iteration. */
 for(int i = 0; i < model.size() ; i++) {

   /* Assumes name ~ Fn(arg1, arg2, ..) */

   ModelFormula exp = (ModelFormula) model.elementAt(i);
   List l = (List) exp.structural();
   FunctionTransformationExpression f = (FunctionTransformationExpression) l.elementAt(0);
   Name n = (FormulaName) f.qualifier();

   List inArgs = f.args();
   List args = new List(inArgs.size() + 1);
    args.addElement(exp.response());
    for(int j = 0; j <  inArgs.size() ; j++)
      args.addElement(inArgs.elementAt(j));

   expressions.addElement(new CachingMethodCall(n, "computeProbability", args));
 }

return(expressions);
}


// line 218 "ModelEvaluator.jweb"
public double computeLikelihood() throws Throwable {
  List el = (List) likelihood().eval(this);
  double total = 1;

  for(Enumeration e = el.elements(); e.hasMoreElements() ;)
   total *= ((Double) e.nextElement()).doubleValue();

 return(total);
}

// line 170 "ModelEvaluator.jweb"
public void computeProposal() 
{
  ModelFormula f;
  /* All the variables for which we want to generate a value in the proposal step. */
  for(Enumeration e = proposalModel().elements() ; e.hasMoreElements() ;) 
  {
    f = (ModelFormula) e.nextElement();
    System.err.println("Trying to handle line : " + f + " ." );
    FunctionTransformationExpression ft = (FunctionTransformationExpression) ((List) f.structural()).elementAt(0);
    MethodCall call = new CachingMethodCall( ft.qualifier(), "generate", ft.args() );
    //    call.qualifier(new Name(call.methodName()));
    //    call.methodName("generate");

    Object value;

    System.err.println("Trying to evaluate the expression: " + call + "." );
    System.err.println("Argument evaulation was " + call.evalArgs() + "." );
    call.evalArgs(true);

    try
      {
	value = call.eval(this);
      }
    catch( Throwable ex )
      {
	ex.printStackTrace();
	throw new RuntimeException("Failed to evaluate call " + call + ".");
      }
    /* assumes name is of length 1. 
       if not, evaluate it in an assignment.
       new AssignExpression(name, new ConstantExpression(value)).eval(this);
     */
    try
      {
	((MCMCStateDatabase) databases.elementAt(PROPOSAL)).assign( ((Name) f.response()).asString(), value);  
      }
    catch( Throwable ex )
      {
	ex.printStackTrace();
	throw new RuntimeException("Failed to make assignement to " + f.response() + ".");
      }

  }
} 


// line 18 "ModelEvaluator.jweb"
}
